# Load binary
load FFMPEG_for_TCL.dll

package require Tk
package require img::raw

# Initialize the library
ffmpeg_init
FFMPEG_FSOUND_Init

#_________________________________________________________________________________________________________________________________
# Define some procedures for playing the video
#_________________________________________________________________________________________________________________________________
proc Open_video {f_name audio_canal tk_img} {
	set ffmpeg_id [FFMPEG_Open_video_stream $f_name]
	set error_message [FFMPEG_get_error_message $ffmpeg_id]
	if {$error_message != ""} {
		 FFMPEG_Info_for_sound_Drain_all $ffmpeg_id
		 FFMPEG_stopAcquisition          $ffmpeg_id
		 FFMPEG_Close_video_stream		 $ffmpeg_id

		 error "Error with stream $ffmpeg_id :\n$error_message"
		}
		
	FFMPEG_set_Synchronisation_threshold $ffmpeg_id 0.5
	set video_buffer_size [FFMPEG_startAcquisition $ffmpeg_id]
	set tx [FFMPEG_Width  $ffmpeg_id]
	set ty [FFMPEG_Height $ffmpeg_id]
	$tk_img configure -width $tx -height $ty

	set ffmpeg_numFrames  [FFMPEG_numFrames    $ffmpeg_id]
	set ffmpeg_frame_rate [FFMPEG_getFramerate $ffmpeg_id]
	
	set binary_buffer [FFMPEG_Get_a_buffer $video_buffer_size]

	# Sound
	set sample_rate [FFMPEG_Sound_sample_rate $ffmpeg_id]
	set cb_audio    [Get_FFMPEG_FMOD_Stream_Info_audio]
	set buf_len 	[expr int(2 * [FFMPEG_Nb_channels $ffmpeg_id] * $sample_rate / [FFMPEG_getFramerate $ffmpeg_id])]
    if {[FFMPEG_Nb_channels $ffmpeg_id] == 2} {set mono_stereo [FFMPEG_FSOUND_Stereo]} else {set mono_stereo [FFMPEG_FSOUND_Mono]}
    set FMOD_audio_stream [FFMPEG_Get_a_new_FSOUND_STREAM $cb_audio \
														  $audio_canal \
														  $buf_len \
														  [expr $mono_stereo | [FFMPEG_FSOUND_signed] | [FFMPEG_FSOUND_16b]] \
														  $sample_rate \
														  [FFMPEG_Info_for_sound_CB $ffmpeg_id] \
						  ]
	
	
	Update_img $ffmpeg_id $binary_buffer $FMOD_audio_stream $tk_img $ffmpeg_frame_rate $tx $ty [clock milliseconds] 0
	return [list $ffmpeg_id $binary_buffer]
}

set stop_playing_video 0
#_________________________________________________________________________________________________________________________________
proc Close_video {ffmpeg_id} {
	global stop_playing_video
	set stop_playing_video 1
}

set is_updating 0
#_________________________________________________________________________________________________________________________________
proc Update_img {ffmpeg_id binary_buffer FMOD_audio_stream tk_img frame_rate tx ty start_time_ms current_frame_num} {
	global is_updating
	global stop_playing_video
	
	if {$is_updating} {return}
	set is_updating 1
	
	if {$stop_playing_video} {
		 puts -nonewline "Stoping video ..."
		 FFMPEG_Info_for_sound_Drain_all $ffmpeg_id; puts -nonewline "sound drained..."
		 FFMPEG_close_FSOUND_STREAM      $FMOD_audio_stream
		 FFMPEG_stopAcquisition          $ffmpeg_id; puts -nonewline "acquisition stopped..."
		 FFMPEG_Close_video_stream		 $ffmpeg_id; puts -nonewline "stream closed..."
		 set stop_playing_video 0
		 puts "DONE"
		 set is_updating 0
		 return
		}
	
	set ms [clock milliseconds]; 
	set num [expr int(($ms - $start_time_ms)*$frame_rate/1000.0)]
	if {$num != $current_frame_num} {
		 set current_frame_num $num
		 FFMPEG_Lock     $ffmpeg_id
		 FFMPEG_getImage $ffmpeg_id $binary_buffer
		 FFMPEG_UnLock   $ffmpeg_id
		 FFMPEG_set_Video_pts_for_audio $ffmpeg_id [FFMPEG_get_Video_pts $ffmpeg_id]
		
		 set buf_name buf_video_for_Video_PM_P_TK_of_$ffmpeg_id
		 global $buf_name

		 set $buf_name "Magic=RAW\nWidth=$tx\nHeight=$ty\nNumChan=3\nByteOrder=Intel\nScanOrder=TopDown\nPixelType=byte\n"
		 FFMPEG_Convert_void_to_binary_tcl_var $binary_buffer [expr 3 * $tx * $ty] $buf_name 1
		 $tk_img put [subst $$buf_name] -format "raw -max 255 -nomap 1"
		}
	set is_updating 0
	after 5 [list Update_img $ffmpeg_id $binary_buffer $FMOD_audio_stream $tk_img $frame_rate $tx $ty $start_time_ms $current_frame_num]
}

#_________________________________________________________________________________________________________________________________
proc go_to_frame {ffmpeg_id num binary_buffer} {
   FFMPEG_Lock       $ffmpeg_id
   FFMPEG_getImageNr $ffmpeg_id $num $binary_buffer
   FFMPEG_UnLock     $ffmpeg_id
}

#_________________________________________________________________________________________________________________________________
#_________________________________________________________________________________________________________________________________
#_________________________________________________________________________________________________________________________________
set img_video [image create photo -format "raw -max 255 -nomap 1"]
label .lab_video -image $img_video
pack .lab_video

puts "lassign \[Open_video VIDEO_FILE 0 \$img_video\] ffmpeg_id binary_buffer\ngo_to_frame \$ffmpeg_id 100 \$binary_buffer"
